import numpy as np
from scipy.optimize import minimize
import lumapi

# open MODE Solutions and load the prerun file
mode = lumapi.MODE(hide=False)
mode.load("EME_optimize_bend.lms")

# evaluate the figure of merit function so it is in the script workspace
mode.feval("bend_fom_function.lsf")

# NOTE, UNITS FOR LENGTHS ARE IN MICRONS HERE

# the FOM function simply calls the underlying fom
def fom(params):
    mode.putv("params",params);
    mode.putv("N",21);
    mode.eval("res = get_fom_T_length(N,params,1.5e-6,1.6e-6);")
    res = mode.getv("res")
    fom = res["fom"]
    return fom

# numerically calculate the fom graident with a 1 nm forward derivative
def fom_grad(cell_spans):
    res0 = fom(cell_spans)
    der = np.zeros_like(cell_spans)
    cellshift = cell_spans.copy()
    i=0
    dx = 1e-3 # this is 1nm as units are microns
    while( i < len(cell_spans)):
        cellshift[i] = cell_spans[i] + dx # units of microns
        der[i] = (fom(cellshift)-res0)/dx
        cellshift[i] = cell_spans[i]
        i += 1
    return der


# number of cells
N = 21

# start with a length of 255 microns. UNITS ARE MICRONS
params = mode.matrix((N-3)/2) + 0.0612245
#define bounds
bnds = [(0., 1.)]*(len(params))
# call optimizer
res = minimize(fun=fom, x0=params, bounds = bnds, method='L-BFGS-B', jac=fom_grad,options={'disp': True,'maxiter' : 8})
print(res.x)

# push to MODE and evaluate final results
params = res.x
mode.putv("params",params);
mode.putv("N",N);
mode.eval("res = get_fom_T_length(N,params,1.5e-6,1.6e-6);")
mode.eval('''
    plot(res.cell_position*1e6,res.radius*1e6,"length (microns)","radius of curvature (microns)","","plot points");
    setplot("show legend",false);
    ''')
res = mode.getv("res")
fom = res["fom"]
T = res["T"]
T_TM = res["T_TM"]
L = res["L"]
print(fom)
print(T)
print(T_TM)
print(L*1e6)

# update monitors so we can see fields and monitors with final best FOM
#mode.setemeanalysis("update monitors",True);
#mode.emepropagate();
# plot the cell spans
#mode.eval("plot(1:length(cell_spans),cell_spans*1e6,'cell number','span (microns)');")




